@@ -7,7 +7,7 @@
 #
 # @author Davide Brunato <brunato@sissa.it>
 #
-__version__ = '2.0.1'
+__version__ = '2.1.0'
 __author__ = "Davide Brunato"
 __contact__ = "brunato@sissa.it"
 __copyright__ = "Copyright 2018-2020, SISSA"
@@ -21,17 +21,20 @@ from .exceptions import ElementPathError, MissingContextError, \
     ElementPathOverflowError, ElementPathZeroDivisionError
 
 from .xpath_context import XPathContext, XPathSchemaContext
-from .xpath_nodes import AttributeNode, TextNode, TypedAttribute, TypedElement, NamespaceNode
+from .xpath_nodes import XPathNode, AttributeNode, TextNode, \
+    NamespaceNode, TypedElement, TypedAttribute
 from .xpath_token import XPathToken
-from .xpath1_parser import XPath1Parser
-from .xpath2_parser import XPath2Parser
+from .xpath1 import XPath1Parser
+from .xpath2 import XPath2Parser
 from .xpath_selectors import select, iter_select, Selector
 from .schema_proxy import AbstractSchemaProxy
+from .regex import RegexError, translate_pattern
+
 
 __all__ = ['ElementPathError', 'MissingContextError', 'ElementPathSyntaxError',
            'ElementPathKeyError', 'ElementPathLocaleError', 'ElementPathNameError',
            'ElementPathOverflowError', 'ElementPathValueError', 'ElementPathTypeError',
            'ElementPathZeroDivisionError', 'datatypes', 'XPathContext', 'XPathSchemaContext',
-           'AttributeNode', 'TextNode', 'TypedAttribute', 'TypedElement', 'NamespaceNode',
-           'XPathToken', 'XPath1Parser', 'XPath2Parser', 'select', 'iter_select', 'Selector',
-           'AbstractSchemaProxy']
+           'XPathNode', 'AttributeNode', 'TextNode', 'NamespaceNode', 'TypedAttribute',
+           'TypedElement', 'XPathToken', 'XPath1Parser', 'XPath2Parser', 'select',
+           'iter_select', 'Selector', 'AbstractSchemaProxy', 'RegexError', 'translate_pattern']
