@@ -25,7 +25,6 @@ import io
 import math
 import pickle
 from decimal import Decimal
-from collections import namedtuple
 from xml.etree import ElementTree
 
 try:
@@ -319,8 +318,8 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
     def test_node_types(self):
         document = self.etree.parse(io.StringIO(u'<A/>'))
         element = self.etree.Element('schema')
-        attribute = 'id', '0212349350'
-        namespace = namedtuple('Namespace', 'prefix uri')('xs', 'http://www.w3.org/2001/XMLSchema')
+        attribute = AttributeNode('id', '0212349350')
+        namespace = NamespaceNode('xs', 'http://www.w3.org/2001/XMLSchema')
         comment = self.etree.Comment('nothing important')
         pi = self.etree.ProcessingInstruction('action', 'nothing to do')
         text = TextNode('aldebaran')
@@ -1013,6 +1012,11 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
         context = XPathContext(root, item=root[0][0])
         self.check_value('/A', [], context=context)
 
+    def test_path_step_operator_with_duplicates(self):
+        root = self.etree.XML('<A>10<B a="2">11</B>10<B a="2"/>10<B>11</B></A>')
+
+        self.check_selector('/A/node()', root, ['10', root[0], '10', root[1], '10', root[2]])
+
     def test_context_item_expression(self):
         root = self.etree.XML('<A><B1><C/></B1><B2/><B3><C1/><C2/></B3></A>')
         self.check_selector('.', root, [root])
