@@ -20,8 +20,8 @@ except ImportError:
     lxml_etree = None
 
 from elementpath import XPathContext, AttributeNode, TypedAttribute
-from elementpath.datatypes import Timezone, DateTime, GregorianDay, \
-    GregorianMonth, GregorianMonthDay, GregorianYear10, GregorianYearMonth10, \
+from elementpath.datatypes import Timezone, DateTime10, DateTime, DateTimeStamp, \
+    GregorianDay, GregorianMonth, GregorianMonthDay, GregorianYear10, GregorianYearMonth10, \
     Duration, YearMonthDuration, DayTimeDuration, Date10, Time, QName, UntypedAtomic
 from elementpath.namespaces import XSD_NAMESPACE
 
@@ -39,6 +39,7 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:string(" hello  ")', ' hello  ')
         self.check_value('xs:string("\thello \n")', '\thello \n')
         self.check_value('xs:string(())', [])
+        self.wrong_syntax('xs:string(()', 'XPST0017')
 
     def test_normalized_string_constructor(self):
         self.check_value('xs:normalizedString("hello")', "hello")
@@ -137,6 +138,7 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         qname = QName(XSD_NAMESPACE, 'xs:element')
         self.check_value('xs:QName(())', [])
         self.check_value('xs:QName("xs:element")', qname)
+        self.check_value('xs:QName(xs:QName("xs:element"))', qname)
         self.check_value('xs:QName(xs:untypedAtomic("xs:element"))', qname)
         self.wrong_type('xs:QName(5)', 'XPTY0004', "the argument has an invalid type")
         self.wrong_value('xs:QName("1")', 'FORG0001', "invalid value")
@@ -211,6 +213,7 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:long("-20")', -20)
         self.wrong_value('xs:int("-20 91")')
         self.wrong_value('xs:int("2147483648")')
+        self.wrong_value('xs:int(xs:untypedAtomic("INF"))')
         self.check_value('xs:int("2147483647")', 2**31 - 1)
         self.check_value('xs:int("-2147483648")', -2**31)
         self.wrong_value('xs:short("40000")')
@@ -232,11 +235,13 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:unsignedByte("128")', 128)
 
     def test_decimal_constructors(self):
-        self.wrong_value('xs:decimal("hello")')
         self.check_value('xs:decimal("19")', 19)
         self.check_value('xs:decimal("19")', Decimal)
         self.check_value('xs:decimal(xs:untypedAtomic("19"))', 19)
 
+        self.wrong_value('xs:decimal("hello")', 'FORG0001')
+        self.wrong_value('xs:decimal(xs:float("INF"))', 'FOCA0002')
+
         root = self.etree.XML('<root a="10.3"/>')
         context = XPathContext(root)
         self.check_value('xs:decimal(@a)', Decimal('10.3'), context=context)
@@ -255,62 +260,114 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
     def test_float_constructor(self):
         self.wrong_value('xs:float("..")')
-        self.wrong_value('xs:float("ab")')
+        self.wrong_value('xs:float("ab")', 'FORG0001')
+        self.wrong_value('xs:float("inf")')
+
         self.check_value('xs:float(25.05)', 25.05)
         self.check_value('xs:float(xs:untypedAtomic(25.05))', 25.05)
         self.check_value('xs:float(-0.00001)', -0.00001)
         self.check_value('xs:float(0.00001)', float)
         self.check_value('xs:float("INF")', float('inf'))
         self.check_value('xs:float("-INF")', float('-inf'))
-        self.wrong_value('xs:float("inf")')
 
         root = self.etree.XML('<root a="10.3"/>')
         context = XPathContext(root, item=AttributeNode('a', 10.3))
         self.check_value('xs:float(.)', float, context=context)
         self.check_value('xs:float(.)', 10.3, context=context)
 
+        self.parser._xsd_version = '1.1'
+        try:
+            self.check_value('xs:float(9.001)', 9.001)
+        finally:
+            self.parser._xsd_version = '1.1'
+
     def test_datetime_constructor(self):
-        tz0 = None
         tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
         self.check_value('xs:dateTime(())', [])
         self.check_value('xs:dateTime("1969-07-20T20:18:00")',
-                         DateTime(1969, 7, 20, 20, 18, tzinfo=tz0))
+                         DateTime10(1969, 7, 20, 20, 18))
         self.check_value('xs:dateTime(xs:untypedAtomic("1969-07-20T20:18:00"))',
-                         DateTime(1969, 7, 20, 20, 18, tzinfo=tz0))
+                         DateTime10(1969, 7, 20, 20, 18))
         self.check_value('xs:dateTime("2000-05-10T21:30:00+05:24")',
                          datetime.datetime(2000, 5, 10, hour=21, minute=30, tzinfo=tz1))
         self.check_value('xs:dateTime("1999-12-31T24:00:00")',
-                         datetime.datetime(2000, 1, 1, 0, 0, tzinfo=tz0))
+                         datetime.datetime(2000, 1, 1, 0, 0))
+        self.check_value('xs:dateTime(xs:date("1969-07-20"))',
+                         DateTime10(1969, 7, 20))
+
+        self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime10)
+        with self.assertRaises(AssertionError):
+            self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime)
+
+        self.parser._xsd_version = '1.1'
+        try:
+            self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime(1969, 7, 20))
+            self.check_value('xs:dateTime(xs:date("1969-07-20"))', DateTime)
+        finally:
+            self.parser._xsd_version = '1.0'
 
         self.wrong_value('xs:dateTime("2000-05-10t21:30:00+05:24")')
         self.wrong_value('xs:dateTime("2000-5-10T21:30:00+05:24")')
         self.wrong_value('xs:dateTime("2000-05-10T21:3:00+05:24")')
         self.wrong_value('xs:dateTime("2000-05-10T21:13:0+05:24")')
         self.wrong_value('xs:dateTime("2000-05-10T21:13:0")')
+        self.check_value('xs:dateTime("-25252734927766554-12-31T12:00:00")', OverflowError)
         self.wrong_type('xs:dateTime(50)', 'FORG0006', '1st argument has an invalid type')
+        self.wrong_type('xs:dateTime("2000-05-10T21:30:00", "+05:24")', 'XPST0017')
 
         root = self.etree.XML('<root a="1969-07-20T20:18:00"/>')
         context = XPathContext(root)
-        self.check_value('xs:dateTime(@a)', DateTime(1969, 7, 20, 20, 18), context=context)
+        self.check_value('xs:dateTime(@a)', DateTime10(1969, 7, 20, 20, 18), context=context)
 
-        context.item = AttributeNode('a', str(DateTime(1969, 7, 20, 20, 18)))
-        self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
+        context.item = AttributeNode('a', str(DateTime10(1969, 7, 20, 20, 18)))
+        self.check_value('xs:dateTime(.)', DateTime10(1969, 7, 20, 20, 18), context=context)
 
         context.item = AttributeNode('a', 'true')
         self.check_value('xs:dateTime(.)', ValueError, context=context)
 
-        context.item = DateTime(1969, 7, 20, 20, 18)
-        self.check_value('xs:dateTime(.)', DateTime(1969, 7, 20, 20, 18), context=context)
+        context.item = DateTime10(1969, 7, 20, 20, 18)
+        self.check_value('xs:dateTime(.)', DateTime10(1969, 7, 20, 20, 18), context=context)
+
+    def test_datetimestamp_constructor(self):
+        tz0 = Timezone(datetime.timedelta(hours=7, minutes=0))
+        tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
+        ts = DateTimeStamp(1969, 7, 20, 20, 18, tzinfo=tz0)
+
+        self.assertEqual(self.parser.xsd_version, '1.0')
+        self.wrong_syntax('xs:dateTimeStamp("1969-07-20T20:18:00+07:00")')
+
+        self.parser._xsd_version = '1.1'
+        try:
+            self.check_value('xs:dateTimeStamp(())', [])
+            self.check_value('xs:dateTimeStamp("1969-07-20T20:18:00+07:00")', ts)
+            self.check_value('xs:dateTimeStamp(xs:untypedAtomic("1969-07-20T20:18:00+07:00"))', ts)
+            self.check_value('xs:dateTimeStamp("1969-07-20T20:18:00+07:00") '
+                             'castable as xs:dateTimeStamp', True)
+            self.check_value('xs:untypedAtomic("1969-07-20T20:18:00+07:00") '
+                             'castable as xs:dateTimeStamp', True)
+            self.check_value('xs:dateTime("1969-07-20T20:18:00+07:00") '
+                             'cast as xs:dateTimeStamp', ts)
+
+            self.check_value('xs:dateTimeStamp("2000-05-10T21:30:00+05:24")',
+                             datetime.datetime(2000, 5, 10, hour=21, minute=30, tzinfo=tz1))
+            self.wrong_value('xs:dateTimeStamp("1999-12-31T24:00:00")')
+            self.wrong_value('xs:dateTimeStamp("2000-05-10t21:30:00+05:24")')
+            self.wrong_type('xs:dateTimeStamp("1969-07-20T20:18:00", "+07:00")', 'XPST0017')
+            self.wrong_type('xs:dateTimeStamp("1969-07-20T20:18:00+07:00"', 'XPST0017')
+        finally:
+            self.parser._xsd_version = '1.0'
 
     def test_time_constructor(self):
-        tz0 = None
-        tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
+        tz = Timezone(datetime.timedelta(hours=5, minutes=24))
         self.check_value('xs:time("21:30:00")',
-                         datetime.datetime(2000, 1, 1, 21, 30, tzinfo=tz0))
+                         datetime.datetime(2000, 1, 1, 21, 30))
         self.check_value('xs:time(xs:untypedAtomic("21:30:00"))',
-                         datetime.datetime(2000, 1, 1, 21, 30, tzinfo=tz0))
+                         datetime.datetime(2000, 1, 1, 21, 30))
         self.check_value('xs:time("11:15:48+05:24")',
-                         datetime.datetime(2000, 1, 1, 11, 15, 48, tzinfo=tz1))
+                         datetime.datetime(2000, 1, 1, 11, 15, 48, tzinfo=tz))
+        self.check_value('xs:time(xs:dateTime("1969-07-20T20:18:00"))',
+                         Time(20, 18, 00))
+        self.wrong_value('xs:time("24:00:01")')
 
         root = self.etree.XML('<root a="13:15:39"/>')
         context = XPathContext(root)
@@ -320,15 +377,17 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:time(.)', Time(20, 10, 00), context=context)
 
     def test_date_constructor(self):
-        tz0 = None
-        tz2 = Timezone(datetime.timedelta(hours=-14, minutes=0))
+        tz = Timezone(datetime.timedelta(hours=-14, minutes=0))
 
         self.check_value('xs:date("2017-01-19")',
-                         datetime.datetime(2017, 1, 19, tzinfo=tz0))
+                         datetime.datetime(2017, 1, 19))
         self.check_value('xs:date(xs:untypedAtomic("2017-01-19"))',
-                         datetime.datetime(2017, 1, 19, tzinfo=tz0))
+                         datetime.datetime(2017, 1, 19))
         self.check_value('xs:date("2011-11-11-14:00")',
-                         datetime.datetime(2011, 11, 11, tzinfo=tz2))
+                         datetime.datetime(2011, 11, 11, tzinfo=tz))
+        self.check_value('xs:date(xs:dateTime("1969-07-20T20:18:00"))',
+                         Date10(1969, 7, 20))
+
         self.wrong_value('xs:date("2011-11-11-14:01")')
         self.wrong_value('xs:date("11-11-11")')
         self.check_value('xs:date(())', [])
@@ -361,15 +420,17 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:date(.)', Date10(2017, 10, 2), context=context)
 
     def test_gregorian_day_constructor(self):
-        tz0 = None
-        tz1 = Timezone(datetime.timedelta(hours=5, minutes=24))
+        tz = Timezone(datetime.timedelta(hours=5, minutes=24))
 
         self.check_value('xs:gDay("---30")',
-                         datetime.datetime(2000, 1, 30, tzinfo=tz0))
+                         datetime.datetime(2000, 1, 30))
         self.check_value('xs:gDay(xs:untypedAtomic("---30"))',
-                         datetime.datetime(2000, 1, 30, tzinfo=tz0))
+                         datetime.datetime(2000, 1, 30))
         self.check_value('xs:gDay("---21+05:24")',
-                         datetime.datetime(2000, 1, 21, tzinfo=tz1))
+                         datetime.datetime(2000, 1, 21, tzinfo=tz))
+        self.check_value('xs:gDay(xs:dateTime("1969-07-20T20:18:00"))',
+                         GregorianDay(20))
+
         self.wrong_value('xs:gDay("---32")')
         self.wrong_value('xs:gDay("--19")')
 
@@ -387,6 +448,8 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.wrong_value('xs:gMonth("--9")')
         self.wrong_value('xs:gMonth("-09")')
         self.wrong_value('xs:gMonth("--13")')
+        self.check_value('xs:gMonth(xs:dateTime("1969-07-20T20:18:00"))',
+                         GregorianMonth(7))
 
         root = self.etree.XML('<root a="--11"/>')
         context = XPathContext(root)
@@ -396,15 +459,17 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:gMonth(.)', GregorianMonth(1), context=context)
 
     def test_gregorian_month_day_constructor(self):
-        tz0 = None
-        tz2 = Timezone(datetime.timedelta(hours=-14, minutes=0))
+        tz = Timezone(datetime.timedelta(hours=-14, minutes=0))
 
         self.check_value('xs:gMonthDay("--07-02")',
-                         datetime.datetime(2000, 7, 2, tzinfo=tz0))
+                         datetime.datetime(2000, 7, 2))
         self.check_value('xs:gMonthDay(xs:untypedAtomic("--07-02"))',
-                         datetime.datetime(2000, 7, 2, tzinfo=tz0))
+                         datetime.datetime(2000, 7, 2))
         self.check_value('xs:gMonthDay("--07-02-14:00")',
-                         datetime.datetime(2000, 7, 2, tzinfo=tz2))
+                         datetime.datetime(2000, 7, 2, tzinfo=tz))
+        self.check_value('xs:gMonthDay(xs:dateTime("1969-07-20T20:18:00"))',
+                         GregorianMonthDay(7, 20))
+
         self.wrong_value('xs:gMonthDay("--7-02")')
         self.wrong_value('xs:gMonthDay("-07-02")')
         self.wrong_value('xs:gMonthDay("--07-32")')
@@ -422,9 +487,13 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:gYear("-2004")', GregorianYear10(-2004))
         self.check_value('xs:gYear("-12540")', GregorianYear10(-12540))
         self.check_value('xs:gYear("12540")', GregorianYear10(12540))
+        self.check_value('xs:gYear(xs:dateTime("1969-07-20T20:18:00"))',
+                         GregorianYear10(1969))
+
         self.wrong_value('xs:gYear("84")')
         self.wrong_value('xs:gYear("821")')
         self.wrong_value('xs:gYear("84")')
+        self.check_value('"99999999999999999999999999999" castable as xs:gYear', False)
 
         root = self.etree.XML('<root a="1999"/>')
         context = XPathContext(root)
@@ -438,8 +507,12 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
                          datetime.datetime(2004, 2, 1))
         self.check_value('xs:gYearMonth(xs:untypedAtomic("2004-02"))',
                          datetime.datetime(2004, 2, 1))
+        self.check_value('xs:gYearMonth(xs:dateTime("1969-07-20T20:18:00"))',
+                         GregorianYearMonth10(1969, 7))
+
         self.wrong_value('xs:gYearMonth("2004-2")')
         self.wrong_value('xs:gYearMonth("204-02")')
+        self.check_value('"99999999999999999999999999999-01" castable as xs:gYearMonth', False)
 
         root = self.etree.XML('<root a="1900-01"/>')
         context = XPathContext(root)
@@ -453,6 +526,10 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:duration(xs:untypedAtomic("P3Y5M1D"))', (41, 86400))
         self.check_value('xs:duration("P3Y5M1DT1H")', (41, 90000))
         self.check_value('xs:duration("P3Y5M1DT1H3M2.01S")', (41, Decimal('90182.01')))
+
+        self.check_value('xs:untypedAtomic("P3Y5M1D") castable as xs:duration', True)
+        self.check_value('"P8192912991912Y" castable as xs:duration', False)
+
         self.wrong_value('xs:duration("P3Y5M1X")')
         self.assertRaises(ValueError, self.parser.parse, 'xs:duration(1)')
 
@@ -473,6 +550,10 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:yearMonthDuration("-P15M")', (-15, 0))
         self.check_value('xs:yearMonthDuration("-P20Y18M")',
                          YearMonthDuration.fromstring("-P21Y6M"))
+        self.check_value('xs:yearMonthDuration(xs:duration("P3Y5M"))', (41, 0))
+        self.check_value('xs:untypedAtomic("P3Y5M") castable as xs:yearMonthDuration', True)
+        self.check_value('"P9999999999999999Y" castable as xs:yearMonthDuration', False)
+
         self.wrong_value('xs:yearMonthDuration("-P15M1D")')
         self.wrong_value('xs:yearMonthDuration("P15MT1H")')
         self.wrong_value('xs:yearMonthDuration("P1MT10H")')
@@ -486,16 +567,23 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
 
     def test_day_time_duration_constructor(self):
         self.check_value('xs:dayTimeDuration("-P2DT15H")', DayTimeDuration(seconds=-226800))
+        self.check_value('xs:dayTimeDuration(xs:duration("-P2DT15H"))',
+                         DayTimeDuration(seconds=-226800))
+
         self.check_value('xs:dayTimeDuration("PT240H")', DayTimeDuration.fromstring("P10D"))
         self.check_value('xs:dayTimeDuration("P365D")', DayTimeDuration.fromstring("P365D"))
         self.check_value('xs:dayTimeDuration(xs:untypedAtomic("PT240H"))',
                          DayTimeDuration.fromstring("P10D"))
+        self.check_value('xs:untypedAtomic("PT240H") castable as xs:dayTimeDuration', True)
         self.check_value('xs:dayTimeDuration("-P2DT15H0M0S")',
                          DayTimeDuration.fromstring('-P2DT15H'))
         self.check_value('xs:dayTimeDuration("P3DT10H")', DayTimeDuration.fromstring("P3DT10H"))
         self.check_value('xs:dayTimeDuration("PT1S")', (0, 1))
         self.check_value('xs:dayTimeDuration("PT0S")', (0, 0))
 
+        self.wrong_value('xs:dayTimeDuration("+P3DT10H")', 'FORG0001')
+        self.check_value('xs:dayTimeDuration("P999999999999999D")', OverflowError)
+
         root = self.etree.XML('<root a="P5DT18H"/>')
         context = XPathContext(root)
         self.check_value('xs:dayTimeDuration(@a)', DayTimeDuration(496800), context=context)
@@ -533,6 +621,9 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         self.check_value('xs:base64Binary("ODQ=")', b'ODQ=')
         self.check_value('xs:base64Binary(xs:base64Binary("ODQ="))', b'ODQ=')
         self.check_value('xs:base64Binary("YWJjZWZnaGk=")', b'YWJjZWZnaGk=')
+
+        self.wrong_value('xs:base64Binary("xyz")')
+        self.wrong_value('xs:base64Binary("\u0411")')
         self.wrong_type('xs:base64Binary(1e2)')
         self.wrong_type('xs:base64Binary(1.1)')
 
@@ -559,6 +650,12 @@ class XPath2ConstructorsTest(xpath_test_class.XPathTestCase):
         context.item = UntypedAtomic('true')
         self.check_value('xs:untypedAtomic(.)', UntypedAtomic(True), context=context)
 
+    def test_notation_constructor(self):
+        self.wrong_type('xs:NOTATION()', 'XPST0017')
+        self.wrong_type('xs:NOTATION(()', 'XPST0017')
+        self.wrong_type('xs:NOTATION(())', 'XPST0017',
+                        'no constructor function exists for xs:NOTATION')
+        self.wrong_name('"A120" castable as xs:NOTATION', 'XPST0080')
 
 @unittest.skipIf(lxml_etree is None, "The lxml library is not installed")
 class LxmlXPath2ConstructorsTest(XPath2ConstructorsTest):
